from . import DictSource

class Defaults(DictSource):
    def __init__(self, defaults=None, **kwargs):
        """
        This source is initialized with a dict.

        :param defaults: A dict with configuration keys and values. If
                         any values are dicts, these are turned into
                         nested config objects.
        :type defaults: dict
        """
        super(Defaults, self).__init__(**kwargs)
        if defaults:
            self.source = defaults
            # if not, DictSource.__init__ ensures that self.source is
            # a empty dict



import codecs

import yaml

from . import DictSource

class YAMLFile(DictSource):
    def __init__(self, yamlfilename=None, writable=True, **kwargs):
        """Loads and optionally saves configuration files in YAML
        format. Since YAML (and the library implementing the support,
        PyYAML) has automatic support for typed values, data from this
        source are typed.

        :param yamlfile: The name of a YAML file. Nested
                         sections are turned into nested config objects.
        :type yamlfile: str
        :param writable: Whether changes to the LayeredConfig object
                         that has this YAMLFile object amongst its
                         sources should be saved in the YAML file.
        :type writable: bool

        """


        super(YAMLFile, self).__init__(**kwargs)
        if 'defaults' in kwargs:
            self.source = kwargs['defaults']
        else:
            with codecs.open(yamlfilename) as fp:
                # do we need safe_load?
                self.source = yaml.safe_load(fp.read())
            self.yamlfilename = yamlfilename
            self.dirty = False
        self.writable = writable
        self.encoding = "utf-8"  # not sure this is ever really needed

    def get(self, key):
        ret = super(YAMLFile, self).get(key)
        if isinstance(ret, bytes):
            ret = ret.decode(self.encoding)
        return ret

    def save(self):
        assert not self.parent, "save() should only be called on root objects"
        if self.yamlfilename:
            with open(self.yamlfilename, "w") as fp:
                yaml.dump(self.source, fp, default_flow_style=False)



# -*- coding: utf-8 -*-

__author__ = 'Staffan Malmgren'
__email__ = 'staffan.malmgren@gmail.com'
__version__ = "0.1.0"

from .layeredconfig import LayeredConfig
from .configsource import ConfigSource
from .dictsource import DictSource
from .defaults import Defaults
from .inifile import INIFile
from .jsonfile import JSONFile
from .commandline import Commandline
from .environment import Environment
from .plistfile import PListFile
from .yamlfile import YAMLFile



from collections import defaultdict
try:
    from collections import OrderedDict
except ImportError:  # pragma: no cover
    # if on python 2.6
    from ordereddict import OrderedDict
from six import text_type as str
import sys
from copy import copy

from . import ConfigSource

class Commandline(ConfigSource):

    rest = []
    """The remainder of the command line, containing all parameters that
    couldn't be turned into configuration settings. """
    
    def __init__(self,
                 commandline=None,
                 sectionsep="-",
                 **kwargs):
        """Load configuration from command line options. Any long-style
        parameters are turned into configuration values, and
        parameters containing the section separator (by default
        ``"-"``) are turned into nested config objects
        (i.e. ``--module-parameter=foo`` results in
        ``self.module.parameter == "foo"``.

        :param commandline: Command line arguments, in list form like
                            :py:data:`sys.argv`. If not provided, uses
                            the real :py:data:`sys.argv`.
        :type  commandline: list
        :param sectionsep: An alternate section separator instead of ``-``.
        :type  sectionsep: str

        """
        super(Commandline, self).__init__(**kwargs)
        if commandline is None:
            if kwargs.get("empty"):
                commandline = []
            else:
                commandline = sys.argv
        self.source = OrderedDict()
        self.sectionargvs = defaultdict(list)
        self.sectionsep = sectionsep
        self.rest = []
        for arg in commandline:
            if isinstance(arg, bytes):
                # FIXME: Find out proper way of finding the encoding
                # of argv
                arg = arg.decode("utf-8") 
            if arg.startswith("--"):
                if "=" in arg:
                    (param, value) = arg.split("=", 1)
                else:
                    (param, value) = (arg, True)  # assume bool, not str
                # '--param' => ['param']
                # '--module-param' => ['module','param']
                # Note: parameter names may not contain sectionsep (ie
                # they can't be called "parse-force").
                parts = param[2:].split(sectionsep)
                self._load_commandline_part(parts, value, self.sectionargvs)
            else:
                self.rest.append(arg)

    def _load_commandline_part(self, parts, value, sectionargvs):
        if len(parts) == 1:
            key = parts[0]
            if key in self.source:
                if not isinstance(self.source[key], list):
                    self.source[key] = [self.source[key]]
                self.source[key].append(value)
            else:
                self.source[key] = value
        else:
            (sectionkey) = parts[0]
            # recreate the cmdline -- note that this will turn
            # valueless options into explicit, ie "--foo" =>
            # "--foo=True"
            arg = "--%s=%s" % (self.sectionsep.join(parts[1:]), value)
            sectionargvs[sectionkey].append(arg)

    def subsections(self):
        return self.sectionargvs.keys()
        
    def subsection(self, key):
        return Commandline(self.sectionargvs[key],
                           parent=self,
                           identifier=self.identifier)

    def has(self, k):
        return k in self.source

    def get(self, k):
        # FIXME: run self_type_value using a type information source
        # that we somehow have access to.
        return self.source[k]

    def set(self, k, v):
        self.source[k] = v
       
    def keys(self):
        return self.source.keys()  # or "for k in self.source: yield k"
        # what about subsections?

    def typed(self, key):
        # if the value is anything other than a string, we can be sure
        # that it contains useful type information (eg bool, list)
        try:
            return not isinstance(self.get(key), str)
        except:
            return False



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import unicode_literals
"""
test_layeredconfig
----------------------------------

Tests for `layeredconfig` module.
"""

import os
import logging
import sys
if sys.version_info < (2, 7, 0):  # pragma: no cover
    import unittest2 as unittest
else: 
    import unittest
import codecs
from six import text_type as str
from datetime import date, datetime
try:
    from collections import OrderedDict
except ImportError:  # pragma: no cover
    # if on python 2.6
    from ordereddict import OrderedDict

# The system under test
from layeredconfig import (LayeredConfig, Defaults, INIFile, JSONFile,
                           YAMLFile, PListFile, Environment, Commandline)


class TestConfigSourceHelper(object):
    # First, a number of straightforward tests for any
    # ConfigSource-derived object. Concrete test classes should set up
    # self.simple and self.complex instances to match these.
    def test_keys(self):
        self.assertEqual(set(self.simple.keys()),
                         set(('home', 'processes', 'force',
                              'extra', 'expires', 'lastrun')))
        self.assertEqual(set(self.complex.keys()),
                         set(('home', 'processes', 'force', 'extra')))

    def test_subsection_keys(self):
        self.assertEqual(set(self.complex.subsection('mymodule').keys()),
                         set(('force', 'extra', 'expires')))

    def test_subsections(self):
        self.assertEqual(set(self.simple.subsections()),
                         set())
        self.assertEqual(set(self.complex.subsections()),
                         set(('mymodule', 'extramodule')))

    def test_subsection_nested(self):
        subsec = self.complex.subsection('mymodule')
        self.assertEqual(set(subsec.subsections()),
                         set(('arbitrary',)))
        
    def test_has(self):
        for key in self.simple.keys():
            self.assertTrue(self.simple.has(key))

    def test_typed(self):
        for key in self.simple.keys():
            self.assertTrue(self.simple.typed(key))
                
    def test_get(self):
        self.assertEqual(self.simple.get("home"), "mydata")
        self.assertEqual(self.simple.get("processes"), 4)
        self.assertEqual(self.simple.get("force"), True)
        self.assertEqual(self.simple.get("extra"), ['foo', 'bar'])
        self.assertEqual(self.simple.get("expires"), date(2014, 10, 15))
        self.assertEqual(self.simple.get("lastrun"),
                         datetime(2014, 10, 15, 14, 32, 7))


class TestLayeredConfigHelper(object):
    # Then, two validation helpers for checking a complete
    # LayeredConfig object, where validation can be performed
    # different depending on the abilities of the source (eg. typing)
    def _test_mainsection(self, cfg,
                          int_type=int,
                          bool_type=bool,
                          list_type=list,
                          date_type=date,
                          datetime_type=datetime):

        self.assertIs(type(cfg.home), str)
        self.assertEqual(cfg.home, 'mydata')
        self.assertIs(type(cfg.processes), int_type)
        self.assertEqual(cfg.processes, int_type(4))
        self.assertIs(type(cfg.force), bool_type)
        self.assertEqual(cfg.force, bool_type(True))
        self.assertIs(type(cfg.extra), list_type)
        if list_type == str:
            want = "foo, bar"  # recommended serialization of lists of strings
        else:
            want = ['foo', 'bar']
        self.assertEqual(cfg.extra, want)
        self.assertIs(type(cfg.expires), date_type)
        if date_type == str:
            self.assertEqual(cfg.expires, date_type(date(2014, 10, 15)))
        else:
            self.assertEqual(cfg.expires, date(2014, 10, 15))
        self.assertIs(type(cfg.lastrun), datetime_type)
        if datetime_type == str:
            self.assertEqual(cfg.lastrun,
                             datetime_type(datetime(2014, 10, 15, 14, 32, 7)))
        else:
            self.assertEqual(cfg.lastrun, datetime(2014, 10, 15, 14, 32, 7))

    def _test_subsections(self, cfg,
                          int_type=int,
                          bool_type=bool,
                          list_type=list,
                          date_type=date,
                          datetime_type=datetime,
                          arbitrary_nesting=True):
        self.assertEqual(cfg.home, 'mydata')
        with self.assertRaises(AttributeError):
            cfg.mymodule.home
        self.assertEqual(cfg.processes, int_type(4))
        with self.assertRaises(AttributeError):
            cfg.mymodule.processes
        self.assertEqual(cfg.force, bool_type(True))
        self.assertEqual(cfg.mymodule.force, bool_type(False))
        if list_type == str:
            want = "foo, bar"  # recommended serialization of lists of strings
        else:
            want = ['foo', 'bar']
        self.assertEqual(cfg.extra, want)
        if list_type == str:
            want = "foo, baz"
        else:
            want = ['foo', 'baz']
        self.assertEqual(cfg.mymodule.extra, want)
        # not supported for INIFile
        if arbitrary_nesting:
            self.assertEqual(cfg.mymodule.arbitrary.nesting.depth, 'works')
        with self.assertRaises(AttributeError):
            cfg.expires
        if date_type == str:
            self.assertEqual(cfg.mymodule.expires,
                             date_type(date(2014, 10, 15)))
        else:
            self.assertEqual(cfg.mymodule.expires, date(2014, 10, 15))


# common helper
class TestINIFileHelper(object):

    def setUp(self):
        super(TestINIFileHelper, self).setUp()
        with open("simple.ini", "w") as fp:
            fp.write("""
[__root__]
home = mydata
processes = 4
force = True
extra = foo, bar
expires = 2014-10-15
lastrun = 2014-10-15 14:32:07
""")

        with open("complex.ini", "w") as fp:
            fp.write("""
[__root__]
home = mydata
processes = 4
force = True
extra = foo, bar

[mymodule]
force = False
extra = foo, baz
expires = 2014-10-15

[extramodule]
unique = True
""")

    def tearDown(self):
        super(TestINIFileHelper, self).tearDown()
        os.unlink("simple.ini")
        os.unlink("complex.ini")


class TestDefaults(unittest.TestCase, TestConfigSourceHelper, TestLayeredConfigHelper):

    simple = Defaults({'home': 'mydata',
                       'processes': 4,
                       'force': True,
                       'extra': ['foo', 'bar'],
                       'expires': date(2014, 10, 15),
                       'lastrun': datetime(2014, 10, 15, 14, 32, 7)})

    complex = Defaults({'home': 'mydata',
                        'processes': 4,
                        'force': True,
                        'extra': ['foo', 'bar'],
                        'mymodule': {'force': False,
                                     'extra': ['foo', 'baz'],
                                     'expires': date(2014, 10, 15),
                                     'arbitrary': {
                                         'nesting': {
                                             'depth': 'works'
                                         }
                                     }
                                 },
                        'extramodule': {'unique': True}})

    def test_config(self):
        cfg = LayeredConfig(self.simple)
        self._test_mainsection(cfg)

    def test_config_subsections(self):
        cfg = LayeredConfig(self.complex)
        self._test_subsections(cfg)


class TestINIFile(TestINIFileHelper, unittest.TestCase,
                  TestConfigSourceHelper,
                  TestLayeredConfigHelper):

    def setUp(self):
        super(TestINIFile, self).setUp()
        self.simple = INIFile("simple.ini")
        self.complex = INIFile("complex.ini")

    # Overrides of TestHelper.test_get, .test_typed and
    # .test_subsection_nested due to limitations of INIFile
    # INIFile carries no typing information
    def test_get(self):
        self.assertEqual(self.simple.get("home"), "mydata")
        self.assertEqual(self.simple.get("processes"), "4")
        self.assertEqual(self.simple.get("force"), "True")
        self.assertEqual(self.simple.get("extra"), "foo, bar")
        self.assertEqual(self.simple.get("expires"), "2014-10-15")
        self.assertEqual(self.simple.get("lastrun"), "2014-10-15 14:32:07")

    def test_typed(self):
        for key in self.simple.keys():
            self.assertFalse(self.simple.typed(key))

    # INIFile doesn't support nested subsections
    def test_subsection_nested(self):
        subsec = self.complex.subsection('mymodule')
        self.assertEqual(set(subsec.subsections()),
                         set(()))

    def test_inifile(self):
        cfg = LayeredConfig(self.simple)
        self._test_mainsection(cfg,
                               int_type=str,
                               bool_type=str,
                               list_type=str,
                               date_type=str,
                               datetime_type=str)

    def test_inifile_subsections(self):
        cfg = LayeredConfig(self.complex)
        self._test_subsections(cfg,
                               int_type=str,
                               bool_type=str,
                               list_type=str,
                               date_type=str,
                               datetime_type=str,
                               arbitrary_nesting=False)

    def test_inifile_default_as_root(self):
        # using a rootsection named DEFAULT triggers different
        # cascading-like behaviour in configparser.

        # load a modified version of complex.ini
        with open("complex.ini") as fp:
            ini = fp.read()
            
        with open("complex-otherroot.ini", "w") as fp:
            fp.write(ini.replace("[__root__]", "[DEFAULT]"))
        cfg = LayeredConfig(INIFile("complex-otherroot.ini",
                                    rootsection="DEFAULT"))

        # this is a modified/simplified version of ._test_subsections
        self.assertEqual(cfg.home, 'mydata')
        self.assertEqual(cfg.processes, '4')
        self.assertEqual(cfg.force, 'True')
        self.assertEqual(cfg.mymodule.force, 'False')
        self.assertEqual(cfg.extra, "foo, bar")
        self.assertEqual(cfg.mymodule.extra, "foo, baz")
        with self.assertRaises(AttributeError):
            cfg.expires
        self.assertEqual(cfg.mymodule.expires, "2014-10-15")

        # this is really unwanted cascading behaviour
        self.assertEqual(cfg.mymodule.home, 'mydata')
        self.assertEqual(cfg.mymodule.processes, '4')

        os.unlink("complex-otherroot.ini")


    def test_inifile_nonexistent(self):
        logging.getLogger().setLevel(logging.CRITICAL)
        cfg = LayeredConfig(INIFile("nonexistent.ini"))
        self.assertEqual([], list(cfg))

        # make sure a nonexistent inifile doesn't interfere with the
        # rest of the LayeredConfig object
        defobj = Defaults({'datadir': 'something'})
        iniobj = INIFile("nonexistent.ini")
        cfg = LayeredConfig(defobj, iniobj)
        self.assertEqual("something", cfg.datadir)

        # and make sure it's settable (should set up the INIFile
        # object and affect it, and leave the defaults dict untouched
        # as it's the lowest priority)
        cfg.datadir = "else"
        self.assertEqual("else", cfg.datadir)
        self.assertEqual("else", iniobj.get("datadir"))
        self.assertEqual("something", defobj.get("datadir"))

        # same as above, but with a "empty" INIFile object
        iniobj = INIFile()
        cfg = LayeredConfig(defobj, iniobj)
        self.assertEqual("something", cfg.datadir)
        cfg.datadir = "else"
        self.assertEqual("else", cfg.datadir)

    def test_write(self):
        cfg = LayeredConfig(INIFile("complex.ini"))
        cfg.mymodule.expires = date(2014, 10, 24)
        # calling write for any submodule will force a write of the
        # entire config file
        LayeredConfig.write(cfg.mymodule)
        want = """[__root__]
home = mydata
processes = 4
force = True
extra = foo, bar

[mymodule]
force = False
extra = foo, baz
expires = 2014-10-24

[extramodule]
unique = True

"""
        with open("complex.ini") as fp:
            got = fp.read().replace("\r\n", "\n")
        self.assertEqual(want, got)

class TestJSONFile(unittest.TestCase, TestConfigSourceHelper,
                   TestLayeredConfigHelper):

    def setUp(self):
        with open("simple.json", "w") as fp:
            fp.write("""
{"home": "mydata",
 "processes": 4,
 "force": true,
 "extra": ["foo", "bar"],
 "expires": "2014-10-15",
 "lastrun": "2014-10-15 14:32:07"}
""")

        with open("complex.json", "w") as fp:
            fp.write("""
{"home": "mydata",
 "processes": 4,
 "force": true,
 "extra": ["foo", "bar"],
 "mymodule": {"force": false,
              "extra": ["foo", "baz"],
              "expires": "2014-10-15",
              "arbitrary": {
                  "nesting": {
                      "depth": "works"
                  }
              }
          },
 "extramodule": {"unique": true}
}
""")
        self.simple = JSONFile("simple.json")
        self.complex = JSONFile("complex.json")

    def tearDown(self):
        os.unlink("simple.json")
        os.unlink("complex.json")

    def test_get(self):
        self.assertEqual(self.simple.get("home"), "mydata")
        self.assertEqual(self.simple.get("processes"), 4)
        self.assertEqual(self.simple.get("force"), True)
        self.assertEqual(self.simple.get("extra"), ['foo', 'bar'])
        self.assertEqual(self.simple.get("expires"), "2014-10-15")
        self.assertEqual(self.simple.get("lastrun"), "2014-10-15 14:32:07")

    def test_typed(self):
        for key in self.simple.keys():
            # JSON can type ints, bools and lists
            if key in ("processes", "force", "extra"):
                self.assertTrue(self.simple.typed(key))
            else:
                self.assertFalse(self.simple.typed(key))

    def test_jsonfile(self):
        cfg = LayeredConfig(self.simple)
        self._test_mainsection(cfg,
                               int_type=int,
                               bool_type=bool,
                               list_type=list,
                               date_type=str,
                               datetime_type=str)

    def test_jsonfile_subsections(self):
        cfg = LayeredConfig(self.complex)
        self._test_subsections(cfg,
                               int_type=int,
                               bool_type=bool,
                               list_type=list,
                               date_type=str,
                               datetime_type=str,
                               arbitrary_nesting=True)

    def test_write(self):
        self.maxDiff = None
        cfg = LayeredConfig(self.complex)
        cfg.mymodule.expires = date(2014, 10, 24)
        # calling write for any submodule will force a write of the
        # entire config file
        LayeredConfig.write(cfg.mymodule)
        want = """{
    "extra": [
        "foo",
        "bar"
    ],
    "extramodule": {
        "unique": true
    },
    "force": true,
    "home": "mydata",
    "mymodule": {
        "arbitrary": {
            "nesting": {
                "depth": "works"
            }
        },
        "expires": "2014-10-24",
        "extra": [
            "foo",
            "baz"
        ],
        "force": false
    },
    "processes": 4
}"""
        with open("complex.json") as fp:
            got = fp.read().replace("\r\n", "\n")
        self.assertEqual(want, got)

class TestYAMLFile(unittest.TestCase,
                   TestConfigSourceHelper,
                   TestLayeredConfigHelper):
    def setUp(self):
        with open("simple.yaml", "w") as fp:
            fp.write("""
home: mydata
processes: 4
force: true
extra: 
- foo
- bar
expires: 2014-10-15
lastrun: 2014-10-15 14:32:07
""")
        with open("complex.yaml", "w") as fp:
            fp.write("""
home: mydata
processes: 4
force: true
extra:
- foo
- bar
mymodule:
    force: false
    extra:
    - foo
    - baz
    expires: 2014-10-15
    arbitrary:
        nesting:
            depth: works
extramodule:
    unique: true
""")
        self.simple = YAMLFile("simple.yaml")
        self.complex = YAMLFile("complex.yaml")

    def tearDown(self):
        os.unlink("simple.yaml")
        os.unlink("complex.yaml")

    # PyYAML seems to transparently convert strings that look like
    # dates, booleans, ints etc into the correct python
    # objects. Wow. 
    def test_yamlfile(self):
        cfg = LayeredConfig(self.simple)
        self._test_mainsection(cfg)

    def test_yamlfile_subsections(self):
        cfg = LayeredConfig(self.complex)
        self._test_subsections(cfg)

    # Also, strings are unicode when they need to be,
    # str otherwise.
    def test_i18n(self):
        with codecs.open("i18n.yaml", "w", encoding="utf-8") as fp:
            fp.write("shrimpsandwich: Räksmörgås")
        cfg = LayeredConfig(YAMLFile("i18n.yaml"))
        self.assertEqual("Räksmörgås", cfg.shrimpsandwich)
        os.unlink("i18n.yaml")

    def test_write(self):
        cfg = LayeredConfig(self.complex)
        cfg.mymodule.expires = date(2014, 10, 24)
        # calling write for any submodule will force a write of the
        # entire config file
        LayeredConfig.write(cfg.mymodule)
        # note that pyyaml sorts keys alphabetically and has specific
        # ideas on how to format the result (controllable through
        # mostly-undocumented args to dump())
        want = """
extra:
- foo
- bar
extramodule:
  unique: true
force: true
home: mydata
mymodule:
  arbitrary:
    nesting:
      depth: works
  expires: 2014-10-24
  extra:
  - foo
  - baz
  force: false
processes: 4
""".lstrip()
        with open("complex.yaml") as fp:
            got = fp.read().replace("\r\n", "\n")
        self.assertEqual(want, got)

class TestPListFile(unittest.TestCase, TestConfigSourceHelper,
                    TestLayeredConfigHelper):

    def setUp(self):
        with open("simple.plist", "w") as fp:
            fp.write("""<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE plist PUBLIC "-//Apple//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
<plist version="1.0">
<dict>
        <key>expires</key>
        <string>2014-10-15</string>
        <key>extra</key>
        <array>
                <string>foo</string>
                <string>bar</string>
        </array>
        <key>force</key>
        <true/>
        <key>home</key>
        <string>mydata</string>
        <key>lastrun</key>
        <date>2014-10-15T14:32:07Z</date>
        <key>processes</key>
        <integer>4</integer>
</dict>
</plist>
""")
        with open("complex.plist", "w") as fp:
            fp.write("""<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE plist PUBLIC "-//Apple//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
<plist version="1.0">
<dict>
        <key>extra</key>
        <array>
                <string>foo</string>
                <string>bar</string>
        </array>
        <key>extramodule</key>
        <dict>
                <key>unique</key>
                <true/>
        </dict>
        <key>force</key>
        <true/>
        <key>home</key>
        <string>mydata</string>
        <key>mymodule</key>
        <dict>
                <key>arbitrary</key>
                <dict>
                        <key>nesting</key>
                        <dict>
                                <key>depth</key>
                                <string>works</string>
                        </dict>
                </dict>
                <key>expires</key>
                <string>2014-10-15</string>
                <key>extra</key>
                <array>
                        <string>foo</string>
                        <string>baz</string>
                </array>
                <key>force</key>
                <false/>
        </dict>
        <key>processes</key>
        <integer>4</integer>
</dict>
</plist>
""")
        self.simple = PListFile("simple.plist")
        self.complex = PListFile("complex.plist")

    def tearDown(self):
        os.unlink("simple.plist")
        os.unlink("complex.plist")

    # override only because plists cannot handle date objects (only datetime)
    def test_get(self):
        self.assertEqual(self.simple.get("home"), "mydata")
        self.assertEqual(self.simple.get("processes"), 4)
        self.assertEqual(self.simple.get("force"), True)
        self.assertEqual(self.simple.get("extra"), ['foo', 'bar'])
        self.assertEqual(self.simple.get("expires"), "2014-10-15")
        self.assertEqual(self.simple.get("lastrun"),
                         datetime(2014, 10, 15, 14, 32, 7))

    def test_plistfile(self):
        cfg = LayeredConfig(self.simple)
        self._test_mainsection(cfg,
                               date_type=str)

    def test_plistfile_subsections(self):
        cfg = LayeredConfig(self.complex)
        self._test_subsections(cfg,
                               date_type=str)

    def test_write(self):
        self.maxDiff = None
        cfg = LayeredConfig(self.complex)
        cfg.mymodule.expires = date(2014, 10, 24)
        # calling write for any submodule will force a write of the
        # entire config file
        LayeredConfig.write(cfg.mymodule)
        # note: plistlib creates files with tabs, not spaces.
        want = """<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE plist PUBLIC "-//Apple//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
<plist version="1.0">
<dict>
	<key>extra</key>
	<array>
		<string>foo</string>
		<string>bar</string>
	</array>
	<key>extramodule</key>
	<dict>
		<key>unique</key>
		<true/>
	</dict>
	<key>force</key>
	<true/>
	<key>home</key>
	<string>mydata</string>
	<key>mymodule</key>
	<dict>
		<key>arbitrary</key>
		<dict>
			<key>nesting</key>
			<dict>
				<key>depth</key>
				<string>works</string>
			</dict>
		</dict>
		<key>expires</key>
		<string>2014-10-24</string>
		<key>extra</key>
		<array>
			<string>foo</string>
			<string>baz</string>
		</array>
		<key>force</key>
		<false/>
	</dict>
	<key>processes</key>
	<integer>4</integer>
</dict>
</plist>
"""
        if sys.version_info < (2,7,0): # pragma: no cover
            # on py26, the doctype includes "Apple Computer" not "Apple"...
            want = want.replace("//Apple//", "//Apple Computer//")
        with open("complex.plist") as fp:
            got = fp.read().replace("\r\n", "\n")
        self.assertEqual(want, got)


class TestCommandline(unittest.TestCase, TestConfigSourceHelper,
                      TestLayeredConfigHelper):

    simple = Commandline(['--home=mydata',
                          '--processes=4',
                          '--force',  # note implicit boolean typing
                          '--extra=foo',
                          '--extra=bar',
                          '--expires=2014-10-15',
                          '--lastrun=2014-10-15 14:32:07'])

    complex = Commandline(['--home=mydata',
                           '--processes=4',
                           '--force=True',
                           '--extra=foo',
                           '--extra=bar',
                           '--mymodule-force=False',
                           '--mymodule-extra=foo',
                           '--mymodule-extra=baz',
                           '--mymodule-expires=2014-10-15',
                           '--mymodule-arbitrary-nesting-depth=works',
                           '--extramodule-unique'])

    # Overrides of TestHelper.test_get, .test_typed and and due to
    # limitations of Commandline (carries almost no typeinfo)
    def test_get(self):
        self.assertEqual(self.simple.get("home"), "mydata")
        self.assertEqual(self.simple.get("processes"), "4")
        self.assertEqual(self.simple.get("force"), True)
        self.assertEqual(self.simple.get("extra"), ['foo','bar']) # note typed!
        self.assertEqual(self.simple.get("expires"), "2014-10-15")
        self.assertEqual(self.simple.get("lastrun"), "2014-10-15 14:32:07")

    def test_typed(self):
        for key in self.simple.keys():
            # these should be typed as bool and list, respectively
            if key in ("force", "extra"):
                self.assertTrue(self.simple.typed(key))
            else:
                self.assertFalse(self.simple.typed(key))                

    def test_commandline(self):
        cfg = LayeredConfig(self.simple)
        self._test_mainsection(cfg,
                               int_type=str,
                               date_type=str,
                               datetime_type=str, 
                               bool_type=bool,  # not always, see below
                               list_type=list)

    def test_commandline_subsections(self):
        cfg = LayeredConfig(self.complex)
        self._test_subsections(cfg,
                               int_type=str,
                               bool_type=str,  # "--mymodule-force=False"
                               list_type=list,
                               date_type=str,
                               datetime_type=str)


class TestEnvironment(unittest.TestCase, TestConfigSourceHelper, TestLayeredConfigHelper):

    simple = Environment({'MYAPP_HOME': 'mydata',
                          'MYAPP_PROCESSES': '4',
                          'MYAPP_FORCE': 'True',
                          'MYAPP_EXTRA': 'foo, bar',
                          'MYAPP_EXPIRES': '2014-10-15',
                          'MYAPP_LASTRUN': '2014-10-15 14:32:07'},
                         prefix="MYAPP_")
    complex = Environment({'MYAPP_HOME': 'mydata',
                           'MYAPP_PROCESSES': '4',
                           'MYAPP_FORCE': 'True',
                           'MYAPP_EXTRA': 'foo, bar',
                           'MYAPP_MYMODULE_FORCE': 'False',
                           'MYAPP_MYMODULE_EXTRA': 'foo, baz',
                           'MYAPP_MYMODULE_EXPIRES': '2014-10-15',
                           'MYAPP_MYMODULE_ARBITRARY_NESTING_DEPTH': 'works',
                           'MYAPP_EXTRAMODULE_UNIQUE': 'True'},
                          prefix="MYAPP_")

    def test_get(self):
        self.assertEqual(self.simple.get("home"), "mydata")
        self.assertEqual(self.simple.get("processes"), "4")
        self.assertEqual(self.simple.get("force"), "True")
        self.assertEqual(self.simple.get("extra"), "foo, bar")
        self.assertEqual(self.simple.get("expires"), "2014-10-15")
        self.assertEqual(self.simple.get("lastrun"), "2014-10-15 14:32:07")

    def test_typed(self):
        for key in self.simple.keys():
            self.assertFalse(self.simple.typed(key))

    def test_environment(self):
        cfg = LayeredConfig(self.simple)
        self._test_mainsection(cfg,
                               int_type=str,
                               bool_type=str,
                               list_type=str,
                               date_type=str,
                               datetime_type=str)

    def test_environment_subsections(self):
        cfg = LayeredConfig(self.complex)
        self._test_subsections(cfg,
                               int_type=str,
                               bool_type=str,
                               list_type=str,
                               date_type=str,
                               datetime_type=str)


class TestTyping(unittest.TestCase, TestLayeredConfigHelper):
    types = {'home': str,
             'processes': int,
             'force': bool,
             'extra': list,
             'mymodule': {'force': bool,
                          'extra': list,
                          'expires': date,
                          'lastrun': datetime,
                      }
             }
    
    def test_typed_commandline(self):
        cmdline = ['--home=mydata',
                   '--processes=4',
                   '--force=True',
                   '--extra=foo',
                   '--extra=bar',
                   '--implicitboolean',
                   '--mymodule-force=False',
                   '--mymodule-extra=foo',
                   '--mymodule-extra=baz',
                   '--mymodule-expires=2014-10-15',
                   '--mymodule-arbitrary-nesting-depth=works',
                   '--extramodule-unique']
        cfg = LayeredConfig(Defaults(self.types), Commandline(cmdline))
        self._test_subsections(cfg)
        self.assertTrue(cfg.implicitboolean)
        self.assertIs(type(cfg.implicitboolean), bool)

    def test_typed_novalue(self):
        # this cmdline only sets some of the settings. The test is
        # that the rest should raise AttributeError (not return None,
        # as was the previous behaviour), and that __iter__ should not
        # include them.
        cmdline = ['--processes=4', '--force=False']
        cfg = LayeredConfig(Defaults(self.types), Commandline(cmdline))
        self.assertEqual(4, cfg.processes)
        self.assertIsInstance(cfg.processes, int)
        with self.assertRaises(AttributeError):
            cfg.home
        with self.assertRaises(AttributeError):
            cfg.extra
        self.assertEqual(set(['processes', 'force']),
                         set(list(cfg)))

    def test_typed_override(self):
        # make sure this auto-typing isn't run for bools
        types = {'logfile': True}
        cmdline = ["--logfile=out.log"]
        cfg = LayeredConfig(Defaults(types), Commandline(cmdline))
        self.assertEqual(cfg.logfile, "out.log")

    def test_typed_commandline_cascade(self):
        # the test here is that __getattribute__ must determine that
        # subconfig.force is not typed in itself, and fetch type
        # information from the root of defaults

        defaults = {'force': True,
                    'lastdownload': datetime,
                    'mymodule': {}}
        cmdline = ['--mymodule-force=False']
        cfg = LayeredConfig(Defaults(defaults), Commandline(cmdline),
                            cascade=True)
        subconfig = getattr(cfg, 'mymodule')
        self.assertIs(type(subconfig.force), bool)
        self.assertEqual(subconfig.force, False)

        # test typed config values that have no actual value. Since
        # they have no value, they should raise AtttributeError
        with self.assertRaises(AttributeError):
            self.assertEqual(cfg.lastdownload, None)
        with self.assertRaises(AttributeError):
            self.assertEqual(subconfig.lastdownload, None)


class TestTypingINIFile(TestINIFileHelper,
                        unittest.TestCase,
                        TestLayeredConfigHelper):
    
    types = {'home': str,
             'processes': int,
             'force': bool,
             'extra': list,
             'mymodule': {'force': bool,
                          'extra': list,
                          'expires': date,
                          'lastrun': datetime,
                      }
             }
    
    def test_typed_inifile(self):
        cfg = LayeredConfig(Defaults(self.types), INIFile("complex.ini"))
        self._test_subsections(cfg, arbitrary_nesting=False)


class TestLayered(TestINIFileHelper, unittest.TestCase):
    def test_layered(self):
        defaults = {'home': 'someplace'}
        cmdline = ['--home=anotherplace']
        env = {'MYAPP_HOME': 'yourdata'}
        cfg = LayeredConfig(Defaults(defaults))
        self.assertEqual(cfg.home, 'someplace')
        cfg = LayeredConfig(Defaults(defaults), INIFile("simple.ini"))
        self.assertEqual(cfg.home, 'mydata')
        cfg = LayeredConfig(Defaults(defaults), INIFile("simple.ini"),
                            Environment(env, prefix="MYAPP_"))
        self.assertEqual(cfg.home, 'yourdata')
        cfg = LayeredConfig(Defaults(defaults), INIFile("simple.ini"),
                            Environment(env, prefix="MYAPP_"),
                            Commandline(cmdline))
        self.assertEqual(cfg.home, 'anotherplace')
        self.assertEqual(['home', 'processes', 'force', 'extra', 'expires',
                          'lastrun'], list(cfg))

    def test_layered_subsections(self):
        defaults = OrderedDict((('force', False),
                                ('home', 'thisdata'),
                                ('loglevel', 'INFO')))
        cmdline = ['--mymodule-home=thatdata', '--mymodule-force']
        cfg = LayeredConfig(Defaults(defaults), Commandline(cmdline),
                            cascade=True)
        self.assertEqual(cfg.mymodule.force, True)
        self.assertEqual(cfg.mymodule.home, 'thatdata')
        self.assertEqual(cfg.mymodule.loglevel, 'INFO')

        # second test is more difficult: the lower-priority Defaults
        # source only contains a subsection, while the higher-priority
        # Commandline source contains no such subsection. Our
        # sub-LayeredConfig object will only have a Defaults source,
        # not a Commandline source (which will cause the
        # __getattribute__ lookup_resource to look in the Defaults
        # object in the sub-LayeredConfig object, unless we do
        # something smart.
        defaults = {'mymodule': defaults}
        cmdline = ['--home=thatdata', '--force']

        o = Commandline(cmdline)
        o.subsection("mymodule").keys()
        cfg = LayeredConfig(Defaults(defaults), Commandline(cmdline),
                            cascade=True)
        self.assertEqual(cfg.mymodule.force, True)
        self.assertEqual(cfg.mymodule.home, 'thatdata')
        self.assertEqual(cfg.mymodule.loglevel, 'INFO')
        self.assertEqual(['force', 'home', 'loglevel'], list(cfg.mymodule))



class TestSubsections(unittest.TestCase):
    def test_list(self):
        defaults = {'home': 'mydata',
                    'subsection': {'processes': 4}}
        cfg = LayeredConfig(Defaults(defaults),
                            cascade=True)
        self.assertEqual(set(['home', 'processes']),
                         set(cfg.subsection))


class TestModifications(TestINIFileHelper, unittest.TestCase):
    def test_modified(self):
        defaults = {'lastdownload': None}
        cfg = LayeredConfig(Defaults(defaults))
        now = datetime.now()
        cfg.lastdownload = now
        self.assertEqual(cfg.lastdownload, now)

    def test_modified_subsections(self):
        defaults = {'force': False,
                    'home': 'thisdata',
                    'loglevel': 'INFO'}
        cmdline = ['--mymodule-home=thatdata', '--mymodule-force']
        cfg = LayeredConfig(Defaults(defaults),
                            INIFile("complex.ini"),
                            Commandline(cmdline),
                            cascade=True)
        cfg.mymodule.expires = date(2014, 10, 24)

    def test_modified_singlesource_subsection(self):
        self.globalconf = LayeredConfig(
            Defaults({'download_text': None,
                      'base': {}}),
            cascade=True)
        # this should't raise an AttributeError
        self.globalconf.base.download_text
        # this shouldn't, either
        self.globalconf.base.download_text = "WHAT"

        

    def test_write_noconfigfile(self):
        cfg = LayeredConfig(Defaults({'lastrun':
                                      datetime(2012, 9, 18, 15, 41, 0)}))
        cfg.lastrun = datetime(2013, 9, 18, 15, 41, 0)
        LayeredConfig.write(cfg)

    def test_set_novalue(self):
        # it should be possible to set values that are defined in any
        # of the configsources, even though only typing information
        # exists there.
        cfg = LayeredConfig(Defaults({'placeholder': int}),
                            Commandline([]))
        cfg.placeholder = 42

        # but it shouldn't be possible to set values that hasn't been
        # defined anywhere.
        with self.assertRaises(AttributeError):
            cfg.nonexistent = 43


class TestAccessors(TestINIFileHelper, unittest.TestCase):

    def test_set(self):
        # a value is set in a particular underlying source, and the
        # dirty flag isn't set.
        cfg = LayeredConfig(INIFile("simple.ini"))
        LayeredConfig.set(cfg, 'expires', date(2013, 9, 18),
                          "inifile")
        # NOTE: For this config, where no type information is
        # available for key 'expires', INIFile.set will convert the
        # date object to a string, at which point typing is lost.
        # Therefore this commmented-out test will fail
        # self.assertEqual(date(2013, 9, 18), cfg.expires)
        self.assertEqual("2013-09-18", cfg.expires)
        self.assertFalse(cfg._sources[0].dirty)

    def test_get(self):
        cfg = LayeredConfig(Defaults({'codedefaults': 'yes',
                                      'force': False,
                                      'home': '/usr/home'}),
                            INIFile('simple.ini'))
        # and then do a bunch of get() calls with optional fallbacks
        self.assertEqual("yes", LayeredConfig.get(cfg, "codedefaults"))
        self.assertEqual("mydata", LayeredConfig.get(cfg, "home"))
        self.assertEqual(None, LayeredConfig.get(cfg, "nonexistent"))
        self.assertEqual("NO!", LayeredConfig.get(cfg, "nonexistent", "NO!"))

if __name__ == '__main__':
    unittest.main()



# this should possibly be a abstract class as well
from . import ConfigSource


class DictSource(ConfigSource):
    def __init__(self, **kwargs):
        """If your backend data is exposable as a python dict, you can
        subclass from this class to avoid implementing :py:meth:`has`,
        :py:meth:`get`, :py:meth:`keys`, :py:meth:`subsection` and
        :py:meth:`subsections`. You only need to write
        :py:meth:`__init__` (which should set ``self.source`` to that
        exposed dict), and possibly :py:meth:`typed` and
        :py:meth:`save`.

        """
        super(DictSource, self).__init__(**kwargs)
        self.source = {}

    def subsections(self):
        for (k, v) in self.source.items():
            if isinstance(v, dict):
                yield k

    def keys(self):
        for (k, v) in self.source.items():
            if not isinstance(v, dict) and not isinstance(v, type):
                yield k

    def subsection(self, key):
        # Make an object of the correct type
        return self.__class__(defaults=self.source[key],
                              parent=self,
                              identifier=self.identifier)

    def typed(self, key):
        # if we have it, we can type it
        return key in self.source        

    def has(self, key):
        # should has return true for types or only for real values?
        return key in self.source and not isinstance(self.source[key], type)

    def get(self, key):
        return self.source[key]

    def set(self, key, value):
        self.source[key] = value



import os

from . import ConfigSource

class Environment(ConfigSource):
    def __init__(self,
                 environ=None,
                 prefix=None,
                 lower=True,
                 sectionsep="_",
                 **kwargs):
        """Loads settings from environment variables. If ``prefix`` is set to
        ``MYAPP_``, the value of the environment variable ``MYAPP_HOME``
        will be available as the configuration setting ``home``.

        :param environ: Environment variables, in dict form like
                        :py:data:`os.environ`. If not provided, uses
                        the real :py:data:`os.environ`.
        :type environ: dict
        :param prefix: Since the entire environment is not suitable to use
                       as a configuration, only variables starting with this
                       prefix are used.
        :type prefix:  str
        :param lower: If true, lowercase the name of environment
                      variables (since these typically uses uppercase)
        :type  lower: True

        """
        super(Environment, self).__init__(**kwargs)
        if environ is None:
            if kwargs.get("empty"):
                environ = {}
            else:
                environ = os.environ
        if prefix is None:
            prefix = ""
        
        self.source = environ
        self.prefix = prefix
        self.sectionsep = sectionsep

    # used by both keys and subsections, but in different ways
    def _internalkeys(self):
        return  [x.lower()[len(self.prefix):] for x in self.source.keys() if x.startswith(self.prefix)]
        
    def keys(self):
        for x in self._internalkeys():
            if self.sectionsep not in x:
                yield x

    def has(self, key):
        # reverse the prefix/lowerize stuff
        k = self.prefix + key.upper()
        return k in self.source

    def get(self, key):
        k = self.prefix + key.upper()
        return self.source[k]

    def set(self, key, val):
        k = self.prefix + key.upper()
        self.source[k] = val

    def typed(self, key):
        return False

    def subsections(self):
        yielded = set()
        for x in self._internalkeys():
            if self.sectionsep in x:
                section = x.split(self.sectionsep)[0]
                if section not in yielded:
                    yield(section)
                    yielded.add(section)

    def subsection(self, key):
        s = key.upper() + self.sectionsep
        newenviron = dict([(k.replace(s,"", 1), v) for k, v in self.source.items() if s in k])
        return Environment(newenviron,
                           prefix=self.prefix,
                           sectionsep=self.sectionsep,
                           parent=self,
                           identifier=self.identifier)



from datetime import datetime
import plistlib
import sys

from six import text_type as str
from six import binary_type as bytes

from . import DictSource


class PListFile(DictSource):
    def __init__(self, plistfilename=None, writable=True, **kwargs):
        """Loads and optionally saves configuration files in PList
        format. Since PList has some support for typed values (supports
        numbers, lists, bools, datetimes *but not dates*), data from
        this source are sometimes typed, sometimes only available as
        strings.

        :param plistfile: The name of a PList file. Nested sections are 
                          turned into nested config objects.
        :type plistfile: str
        :param writable: Whether changes to the LayeredConfig object
                         that has this PListFile object amongst its
                         sources should be saved in the PList file.
        :type writable: bool
        """
        if sys.version_info >= (3,4):
            self.reader = plistlib.load
            self.writer = plistlib.dump
        else:
            self.reader = plistlib.readPlist
            self.writer = plistlib.writePlist
        super(PListFile, self).__init__(**kwargs)
        if 'defaults' in kwargs:
            self.source = kwargs['defaults']
        else:
            with open(plistfilename, "rb") as fp:
                self.source = self.reader(fp)
            self.plistfilename = plistfilename
            self.dirty = False
        self.encoding = "utf-8"  # I hope this is a sensible default
        self.writable = writable
        
    def set(self, key, value):
        # plist natively supports some types but not all (notably not date)
        if not isinstance(value, (str, bool, int, list, datetime)):
            value = str(value)
        super(PListFile, self).set(key, value)

    def get(self, key):
        ret = super(PListFile, self).get(key)
        if isinstance(ret, bytes):
            ret = ret.decode(self.encoding)
        return ret

    def save(self):
        assert not self.parent, "save() should only be called on root objects"
        if self.plistfilename:
            with open(self.plistfilename, "wb") as fp:
                self.writer(self.source, fp)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import unicode_literals
"""
test_layeredconfig
----------------------------------

Tests for `layeredconfig` module.
"""

import os
import sys
if sys.version_info < (2, 7, 0):  # pragma: no cover
    import unittest2 as unittest
else: 
    import unittest
import shutil
import six


class Examples(unittest.TestCase):

    def _test_pyfile(self, pyfile, want=True, comparator=None):
        # temporarily redefine the print function in current context
        l = locals()
        l['print'] = lambda x: None
        with open(pyfile) as fp:
            pycode = compile(fp.read(), pyfile, 'exec')
        result = six.exec_(pycode, globals(), l)
        # the exec:ed code is expected to set return_value
        got = locals()['return_value']
        if not comparator:
            comparator = self.assertEqual
        comparator(want, got)

    def test_firststep(self):
        self._test_pyfile("docs/examples/firststep.py")
        os.unlink("myapp.ini")

    def test_usage(self):
        shutil.copy2("docs/examples/myapp.ini", os.getcwd())
        self._test_pyfile("docs/examples/usage.py")
        os.unlink("myapp.ini")



# -*- coding: utf-8 -*-

import itertools
import logging


try:
    from collections import OrderedDict
except ImportError:  # pragma: no cover
    # if on python 2.6
    from ordereddict import OrderedDict

class LayeredConfig(object):
    def __init__(self, *sources, **kwargs):
        """Creates a config object from one or more sources and provides
        unified access to a nested set of configuration
        parameters. The source of these parameters a config file
        (using .ini-file style syntax), command line parameters, and
        default settings embedded in code. Command line parameters
        override configuration file parameters, which in turn override
        default settings in code (hence **Layered** Config).

        Configuration parameters are accessed as regular object
        attributes, not dict-style key/value pairs.  Configuration
        parameter names should therefore be regular python
        identifiers, and preferrably avoid upper-case and "_" as well
        (i.e. only consist of the characters a-z and 0-9)

        Configuration parameter values can be typed (strings,
        integers, booleans, dates, lists...). Even though some sources
        lack typing information (eg in INI files, command-line
        parameters and enviroment variables, everything is a string),
        LayeredConfig will attempt to find typing information in other
        sources and convert data.

        :param \*sources: Initialized ConfigSource-derived objects
        :param cascade: If an attempt to get a non-existing parameter
                        on a sub (nested) configuration object should
                        attempt to get the parameter on the parent
                        config object. ``False`` by default,
        :type cascade: bool
        :param writable: Whether configuration values should be mutable.
                         ``True`` by default. This does not affect
                         :py:meth:`~Layeredconfig.set`. 
        :type writable: bool

        """
        self._sources = sources
        self._subsections = OrderedDict()
        self._cascade = kwargs.get('cascade', False)
        self._writable = kwargs.get('writable', True)
        self._parent = None
        self._sectionkey = None

        # Each source may have any number of named subsections. We
        # create a LayeredConfig object for each name, and stuff all
        # matching subections from each of our sources in it.
        #
        # 1. find all names
        sectionkeys = []
        for src in self._sources:
            try:
                for k in src.subsections():
                    if k not in sectionkeys:
                        sectionkeys.append(k)
            except AttributeError:  # possibly others, or all
                # we couldn't get any subsections for source, perhaps
                # because it's an "empty" source. Well, that's ok.
                pass
                

        for k in sectionkeys:
            # 2. find all subsections in all of our sources
            s = []
            for src in self._sources:
                if k in list(src.subsections()):
                    s.append(src.subsection(k))
                else:
                    # create an "empty" subsection object. It's
                    # important that all the LayeredConfig objects in a
                    # tree have the exact same set of
                    # ConfigSource-derived types.
                    # print("creating empty %s" % src.__class__.__name__)
                    s.append(src.__class__(parent=src,
                                           identifier=src.identifier,
                                           writable=src.writable,
                                           empty=True,
                                           cascade=self._cascade))
            # 3. create a LayeredConfig object for the subsection
            c = LayeredConfig(*s,
                              cascade=self._cascade,
                              writable=self._writable)
            c._sectionkey = k
            c._parent = self
            self._subsections[k] = c
            
        # cascade=False, writable=True,

    @staticmethod
    def write(config):
        """Commits any pending modifications, ie save a configuration file if
        it has been marked "dirty" as a result of an normal
        assignment. The modifications are written to the first
        writable source in this config object.

        .. note:: 

           This is a static method, ie not a method on any object
           instance. This is because all attribute access on a
           LayeredConfig object is meant to retrieve configuration
           settings. 

        :param config: The configuration object to save
        :type  config: layeredconfig.LayeredConfig

        """
        root = config
        while root._parent:
            root = root._parent

        for source in root._sources:
            if source.writable and source.dirty:
                source.save()

    @staticmethod
    def set(config, key, value, sourceid="defaults"):
        """Sets a value in this config object *without* marking any source
        dirty, and with exact control of exactly where to set the
        value. This is mostly useful for low-level trickery with
        config objects.

        :param config: The configuration object to set values on
        :param key: The parameter name
        :param value: The new value
        :param sourceid: The identifier for the underlying source that the 
                         value should be set on.
        """
        for source in config._sources:
            if source.identifier == sourceid:
                source.set(key, value)
        # What if no source is found? We silently ignore...

    @staticmethod
    def get(config, key, default=None):
        """Gets a value from the config object, or return a default value if
        the parameter does not exist, like :py:meth:`dict.get` does.
        """

        if hasattr(config, key):
            return getattr(config, key)
        else:
            return default

# These are methods i'd like to implement next
#        
#    @staticmethod
#    def where(config, key):
#

        """returns the identifier of a source where a given key is found, or None."""
#        pass
#
#    @staticmethod
#    def dump(config):
#        """Returns the contents of config as a dict."""
#        pass
#
#    @staticmethod
#    def load(config, d):
#        """Recreates a dump()ed config object."""
#        pass

    def __iter__(self):
        l = set()

        iterables = [x.keys() for x in self._sources]

        if self._cascade:
            c = self
            while c._parent:
                iterables.append(c._parent)
                c = c._parent

        for k in itertools.chain(*iterables):
            if k not in l:
                l.add(k)
                yield k

    def __getattr__(self, name):

        if name in self._subsections:
            return self._subsections[name]

        found = False
        # find the appropriate value in the highest-priority source
        for source in reversed(self._sources):
            # if self._cascade, we must climb the entire chain of
            # .parent objects to be sure.
            done = False
            while not done:
                if source.has(name):
                    found = True
                    done = True  # we found it
                elif self._cascade and source.parent:
                    source = source.parent
                else:
                    done = True  # we didn't find it
            if found:
                break

        if found:
            if source.typed(name):
                return source.get(name)
            else:
                # we need to find a typesource for this value. 
                done = False
                this = self
                while not done:
                    for typesource in reversed(this._sources):
                        if typesource.typed(name):
                            done = True
                            break
                    if not done and self._cascade and this._parent:
                        # Iterate up the parent chain to find it.
                        this = this._parent
                    else:
                        done = True

                if typesource.typed(name):
                    return typesource.typevalue(name, source.get(name))
                else:
                    # we can't type this data, return as-is
                    return source.get(name)
        else:
            if self._cascade and self._parent:
                return self._parent.__getattr__(name)
        
        raise AttributeError("Configuration key %s doesn't exist" % name)

    def __setattr__(self, name, value):
        # print("__setattribute__ %s to %s" % (name,value))
        if name.startswith("_"):
            object.__setattr__(self, name, value)
            return

        # we need to get access to two sources:

        # 1. the highest-priority writable source (regardless of
        #    whether it originally had this value)
        found = False
        for writesource in reversed(self._sources):
            if writesource.writable:
                found = True
                break
        if found:
            writesource.set(name, value)
            writesource.dirty = True
            while writesource.parent:
                writesource = writesource.parent
                writesource.dirty = True

        # 2. the highest-priority source that has this value (typed or
        # not) or contains typing info for it.
        found = False
        for source in reversed(self._sources):
            if source.has(name) or source.typed(name):
                found = True
                break
        if found:
            source.set(name, value)  # regardless of typing
        elif self._cascade and self._parent:
            return self._parent.__setattr__(name, value)
        else:
            raise AttributeError("Configuration key %s doesn't exist" % name)




import json
from six import text_type as str

from . import DictSource


class JSONFile(DictSource):

    def __init__(self, jsonfilename=None, writable=True, **kwargs):
        """Loads and optionally saves configuration files in JSON
        format. Since JSON has some support for typed values (supports
        numbers, lists, bools, but not dates or datetimes), data from
        this source are sometimes typed, sometimes only available as
        strings.

        :param jsonfile: The name of a JSON file, whose root element
                         should be a JSON object (python dict). Nested
                         objects are turned into nested config objects.
        :type jsonfile: str
        :param writable: Whether changes to the LayeredConfig object
                         that has this JSONFile object amongst its
                         sources should be saved in the JSON file.
        :type writable: bool

        """
        super(JSONFile, self).__init__(**kwargs)
        if 'defaults' in kwargs:
            self.source = kwargs['defaults']
        else:
            with open(jsonfilename) as fp:
                self.source = json.load(fp)
            self.jsonfilename = jsonfilename
            self.dirty = False
        self.writable = writable

    def typed(self, key):
        # if the value is anything other than a string, we can be sure
        # that it contains useful type information.
        return not isinstance(self.get(key), str)

    def set(self, key, value):
        # simple stringification -- should perhaps only be done in the
        # save step through a method passed as a default parameter to
        # json dumps
        self.source[key] = str(value)

    def save(self):
        assert not self.parent, "save() should only be called on root objects"
        if self.jsonfilename:
            with open(self.jsonfilename, "w") as fp:
                json.dump(self.source, fp, indent=4, separators=(',',': '), sort_keys=True)



import os
import logging
import six
from six.moves import configparser
from six import text_type as str
try:
    from collections import OrderedDict
except ImportError:  # pragma: no cover
    # if on python 2.6
    from ordereddict import OrderedDict

from . import ConfigSource


class INIFile(ConfigSource):
    def __init__(self,
                 inifilename=None,
                 rootsection="__root__",
                 writable=True, 
                 **kwargs):
        """Loads and optionally saves configuration files in INI format, as
        handled by :py:mod:`configparser`.

        :param inifile: The name of a ini-style configuration
                        file. The file should have a top-level
                        section, by default named ``__root__``, whose
                        keys are turned into top-level configuration
                        parameters. Any other sections in this file
                        are turned into nested config objects.
        :type inifile: str
        :param rootsection: An alternative name for the top-level section. 
                            See note below. 
        :type rootsection: str
        :param writable: Whether changes to the LayeredConfig object
                         that has this INIFile object amongst its
                         sources should be saved in the INI file.
        :type writable: bool

        .. note::

           Since this source uses :py:mod:`configparser`, and since
           that module handles sections named ``[DEFAULT]``
           differently, this module will have a sort-of automatic
           cascading feature for subsections if ``DEFAULT`` is used as
           ``rootsection``

        """
        super(INIFile, self).__init__(**kwargs)
        if inifilename:
            if not os.path.exists(inifilename):
                logging.warn("INI file %s does not exist" % inifilename)
                # create a empty RawConfigParser (Raw to avoid the
                # interpolation behaviour of other classes)
                self.source = configparser.RawConfigParser(dict_type=OrderedDict)
                self.inifilename = inifilename
                if rootsection != "DEFAULT":
                    self.source.add_section(rootsection)
            else:
                self.source = configparser.RawConfigParser(dict_type=OrderedDict)
                self.source.read(inifilename)
                self.inifilename = inifilename
        # only used when creating new INIFile objects internally
        elif 'config' in kwargs:  
            self.source = kwargs['config']
            self.inifilename = None
        else:
            # This is an "empty" INIFile object
            self.source = configparser.ConfigParser(dict_type=OrderedDict)
            self.source.add_section(rootsection)
            self.inifilename = None
        if 'section' in kwargs:
            self.sectionkey = kwargs['section']
        else:
            self.sectionkey = rootsection
            self.dirty = False
        self.writable = writable
        self.rootsection = rootsection

    def typed(self, key):
        # INI files carry no intrinsic type information
        return False

    def subsections(self):
        # self.source may be None if we provided the path to a
        # nonexistent inifile (this should probably throw an exception
        # instead)
        if not self.source:
            return []
        elif self.sectionkey != self.rootsection:
            # sections can't be nested using configparser
            return []
        else:
            return [x for x in self.source.sections() if x != self.rootsection]

    def subsection(self, key):
        return INIFile(config=self.source, section=key,
                       parent=self, identifier=self.identifier)

    def has(self, key):
        if self.sectionkey == "DEFAULT":
            return key in self.source.defaults()
        else:
            return self.source.has_option(self.sectionkey, key)

    def get(self, key):
        return str(self.source.get(self.sectionkey, key))

    def set(self, key, value):
        self.source.set(self.sectionkey, key, str(value))

    def keys(self):
        for k in self.source.options(self.sectionkey):
            yield k

    def save(self):
        # this should only be called on root objects
        assert not self.parent, "save() should only be called on root objects"
        if self.inifilename:
            with open(self.inifilename, "w") as fp:
                self.source.write(fp)



from abc import ABCMeta, abstractmethod
from datetime import date, datetime
import ast

class ConfigSource(object):
    __metaclass__ = ABCMeta

    identifier = None
    """A string identifying this source, primarily used with
    :py:meth:`LayeredConfig.set`."""

    writable = False
    """Whether or not this source can accept changed configuration
    settings and store them in the same place as the original setting came
    from."""

    dirty = False
    """For writable sources, whether any parameter value in this source
    has been changed so that a call to :py:meth:`save` might be needed."""

    parent = None
    """The parent of this source, if this represents a nested
    configuration source, or None"""

    source = None
    """By convention, this should be your main connection handle, data
     access object, or other resource neededed to retrieve the
     settings."""

    @abstractmethod  # but subclasses should still call it through super()
    def __init__(self, **kwargs):
        """The constructor of the class should set up needed
        resources, such as opening and parsing a configuration file.

        It is a good idea to keep whatever connection handles, data
        access objects, or other resources needed to retrieve the
        settings, as unprocessed as possible. The methods that
        actually need the data (:py:meth:`has`, :py:meth:`get`,
        :py:meth:`subsection`, :py:meth:`subsections` and possibly
        :py:meth:`typed`) should use those resources directly instead
        of reading from cached locally stored copies.

        The constructor must call the superclass'  ``__init__`` method with all
        remaining keyword arguments, ie. ``super(MySource,
        self).__init__(**kwargs)``.

        """
        
        self.identifier = kwargs.get('identifier',
                                     self.__class__.__name__.lower())
        self.writable = kwargs.get('writable', False)
        self.parent = kwargs.get('parent')
        self.source = None

    @abstractmethod
    def has(self, key):
        """This method should return true if the parameter identified by
        ``key`` is present in this configuration source. It is up to
        each configuration source to define the semantics of what
        exactly "is present" means, but a guideline is that only real
        values should count as being present. If you only have some
        sort of placeholder or typing information for ``key`` this
        should probably not return True.

        Note that it is possible that a configuration source would
        return True for ``typed(some_key)`` and at the same time
        return False for ``has(some_key)``, if the source only carries
        typing information, not real values.

        """
        pass # pragma: no cover

    @abstractmethod
    def get(self, key):
        """Should return the actual value of the parameter identified by
        ``key``. If ``has(some_key)`` returns True, ``get(some_key)``
        should always succeed. If the configuration source does not
        include intrinsic typing information (ie. everything looks
        like a string) this method should return the string as-is,
        LayeredConfig is responsible for converting it to the correct
        type."""
        pass # pragma: no cover

    @abstractmethod
    def keys(self): pass  # pragma: no cover

    @abstractmethod
    def typed(self, key):
        """Should return True if this source contains typing information for
        ``key``, ie information about which data type this parameter
        should be. 

        For sources where everything is stored as a string, this
        should generally return False (no way of distinguishing an
        actual string from a date formatted as a string).
        """
        pass # pragma: no cover

    @abstractmethod
    def subsections(self):
        """Should return a list (or other iterator) of subsection keys, ie
        names that represent subsections of this configuration
        source. Not all configuration sources need to support
        subsections. In that case, this should just return an empty
        list.

        """
        pass  # pragma: no cover

    @abstractmethod
    def subsection(self, key):
        """Should return the subsection identified by ``key``, in the form of
        a new object of the same class, but initialized
        differently. Exactly how will depend on the source, but as a
        general rule the same resource handle used as ``self.source``
        should be passed to the new object. Often, the subsection key
        will need to be provided to the new object as well, so that
        :py:meth:`get` and other methods can use it to look in the
        correct place.

        As a general rule, the constructor should be called with a
        ``parent`` parameter set to ``self``.
        """
        pass  # pragma: no cover

    @abstractmethod
    def set(self, key, value):
        """Should set the parameter identified by ``key`` to the new value
        ``value``.

        This method should be prepared for any type of value, ie ints,
        lists, dates, bools... If the backend cannot handle the given
        type, it should convert to a str itself.

        Note that this does not mean that the changes should be
        persisted in the backend data, only in the existing objects
        view of the data (only when :py:meth:`save` is called, the
        changes should be persisted).
        """
        pass  # pragma: no cover

    def save(self):
        """Persist changed data to the backend. This generally means to update
        a loaded configuration file with all changed data, or similar.

        This method will only ever be called if :py:data:`writable` is
        True, and only if :py:data:`dirty` has been set to True.

        If your source is read-only, you don't have to implement this method.
        """
        pass 

    # @abstractmethod
    # should this be called "coerce", "cast" or something similar
    def typevalue(self, key, value):
        """Given a parameter identified by ``key`` and an untyped string,
        convert that string to the type that our version of key has.

        """

        def boolconvert(value):
            # not all bools should be converted, see test_typed_commandline
            if value == "True":
                return True
            elif value == "False":
                return False
            else:
                return value
            
        def listconvert(value):
            # this function might be called with both string
            # represenations of entire lists and simple (unquoted)
            # strings. String representations come in two flavours,
            # the (legacy/deprecated) python literal (eg "['foo',
            # 'bar']") and the simple (eg "foo, bar") The
            # ast.literal_eval handles the first case, and if the
            # value can't be parsed as a python expression, the second
            # way is attempted. If both fail, it is returned verbatim
            # (not wrapped in a list, for reasons)
            try:
                return ast.literal_eval(value)
            except (SyntaxError, ValueError):
                if "," in value:
                    return [x.strip() for x in value.split(",")]
                else:
                    return value

        def datetimeconvert(value):
            try:
                return datetime.strptime(value, "%Y-%m-%d %H:%M:%S.%f")
            except ValueError:
                return datetime.strptime(value, "%Y-%m-%d %H:%M:%S")

        def dateconvert(value):
            try:
                return datetime.strptime(value, "%Y-%m-%d").date()
            except ValueError:
                return datetime.strptime(value, "%Y-%m-%d").date()

        # self.get(key) should never fail
        default = self.get(key)
        if type(default) == type:
            # print("Using class for %s" % key)
            t = default
        else:
            # print("Using instance for %s" % key)
            t = type(default)

        if t == bool:
            t = boolconvert
        elif t == list:
            t = listconvert
        elif t == date:
            t = dateconvert
        elif t == datetime:
            t = datetimeconvert
        # print("Converting %r to %r" % (value,t(value)))
        return t(value)





#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

badges = open('BADGES.rst').read()
readme = open('README.rst').read()
history = open('HISTORY.rst').read().replace('.. :changelog:', '')

requirements = [
    'six',
    'PyYAML'
]

if sys.version_info < (2, 7, 0):
    requirements.append('ordereddict >= 1.1')

test_requirements = [
    # TODO: put package test requirements here
]


# we can't just "import layeredconfig" to get at
# layeredconfig.__version__ since it might have unmet dependencies at
# this point. Exctract it directly from the file (code from rdflib:s
# setup.py)
def find_version(filename):
    import re
    _version_re = re.compile(r'__version__ = "(.*)"')
    for line in open(filename):
        version_match = _version_re.match(line)
        if version_match:
            return version_match.group(1)

setup(
    name='layeredconfig',
    version=find_version('layeredconfig/__init__.py'),
    description='Manages configuration coming from config files, environment variables, command line arguments, code defaults or other sources',
    long_description=badges + "\n\n" + readme + '\n\n' + history,
    author='Staffan Malmgren',
    author_email='staffan.malmgren@gmail.com',
    url='https://github.com/staffanm/layeredconfig',
    packages=[
        'layeredconfig',
    ],
    package_dir={'layeredconfig':
                 'layeredconfig'},
    include_package_data=True,
    install_requires=requirements,
    license="BSD",
    zip_safe=False,
    keywords='configuration',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ],
    test_suite='tests',
    tests_require=test_requirements
)
